import {
	Injectable,
	OnModuleInit,
	UnauthorizedException,
} from "@nestjs/common";
import { eq } from "drizzle-orm";
import { SignInDto, SignUpDto } from "src/auth/auth.dto";
import { UserUpdateSchema } from "src/auth/auth.schema";
import { CredentialsService } from "src/credentials/credentials.service";
import { DrizzleService } from "src/drizzle/drizzle.service";
import { UsersTable } from "src/schema";
import { IUserUpdateData } from "src/auth/auth.types";

@Injectable()
export class AuthService implements OnModuleInit {
	constructor(
		private db: DrizzleService,
		private credentials: CredentialsService,
	) {}
	async doRegister(data: SignUpDto) {
		console.log(data);
		const existingUser = await this.db
			.use()
			.select()
			.from(UsersTable)
			.where(eq(UsersTable.email, data.email))
			.prepare("userByEmail")
			.execute();
		if (existingUser.length !== 0)
			throw new UnauthorizedException("Already exist");
		const query = await this.db
			.use()
			.insert(UsersTable)
			.values({
				firstName: data.firstName,
				lastName: data.lastName,
				email: data.email,
				hash: await this.credentials.hash(data.password),
			})
			.returning()
			.prepare("insertUser")
			.execute()
			.catch((err) => {
				console.error(err);
				throw new UnauthorizedException(
					"Error occurred while inserting user",
					err,
				);
			});
		return {
			message: "User created, check your email for validation.",
			token: await this.credentials.signAuthToken({ sub: query[0].uuid }),
		};
	}

	async doLogin(data: SignInDto) {
		const user = await this.db
			.use()
			.select()
			.from(UsersTable)
			.where(eq(UsersTable.email, data.email))
			.prepare("userByEmail")
			.execute();
		if (user.length !== 1)
			throw new UnauthorizedException("Invalid credentials");
		const passwordMatch = await this.credentials.check(
			data.password,
			user[0].hash,
		);
		if (!passwordMatch) throw new UnauthorizedException("Invalid credentials");
		const token = await this.credentials.signAuthToken({ sub: user[0].uuid });
		return {
			message: "Login successful",
			token: token,
		};
	}

	async fetchUser() {
		//TODO Pagination
		const usersInDb = await this.db.use().select().from(UsersTable);
		const result = {
			total: usersInDb.length,
			users: usersInDb.map((user) => {
				delete user.hash;
				return {
					...user,
				};
			}),
		};
		console.log(result);
		return result;
	}

	async updateUser(targetId: string, userData: IUserUpdateData) {
		const validationResult = UserUpdateSchema.safeParse(userData);
    if (!validationResult.success) {
      throw new UnauthorizedException(validationResult.error);
    }
    const updateQuery = await this.db
      .use()
      .update(UsersTable)
      .set({
				...userData
			})
      .where(eq(UsersTable.uuid, targetId))
      .prepare("updateUserById")
      .execute()
      .catch((err) => {
        console.error(err);
        throw new UnauthorizedException(
          "Error occurred while updating user",
          err,
        );
      });
    return true;
	}

	async deleteUser(targetId: string) {
		await this.db
			.use()
			.delete(UsersTable)
			.where(eq(UsersTable.uuid, targetId))
			.prepare("deleteUserById")
			.execute()
			.catch((err) => {
				console.error(err);
				throw new UnauthorizedException(
					"Error occurred while deleting user",
					err,
				);
			});
		return true;
	}

	async onModuleInit() {
		setTimeout(() => {
			this.fetchUser();
		}, 2000);
	}
}
